{% extends "base.html" %}
{% block title %}Lektor Admin{% endblock %}
{% block body %}
  <div id=dash>
    <div class=loadbox>
      <p>Loading Interface ...</p>
      <noscript>You must enable JavaScript to use the editing interface.</noscript>
    </div>
  </div>
  <script type=text/javascript>
    var $LEKTOR_CONFIG = {
      admin_root: {{ g.admin_context.admin_root|tojson }},
      site_root: {{ g.admin_context.site_root|tojson }},
      lang: {{ g.admin_context.info.ui_lang|tojson }}
    };
  </script>
{% endblock %}
{% block scripts %}
  {{ super() }}
  <script type=text/javascript src="{{ url_for('static', filename='gen/app.js') }}"></script>
{% endblock %}
